//+------------------------------------------------------------------+
//|                                         Directional_Breakout.mq4 |
//|                               Copyright  2014, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Green
#property indicator_color2 Yellow
#property indicator_color3 Yellow
#property indicator_color4 Red
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2

enum enPrices
{

   pr_close,   // Close
   pr_open,    // Open
   pr_high,    // High
   pr_low,     // Low
   pr_median,  // Median
   pr_typical, // Typical
   pr_weighted // Weighted
};
enum enMaTypes
{
   ma_sma,     // sma (simple moving average)
   ma_ema,     // ema (exponential moving average)
   ma_smma,    // smma (smoothed moving average)
   ma_lwma     // lwma (liner weighted moving average)
};

extern string    TimeFrame = "Current time frame";  // Time frame to use
extern enPrices  Price     = pr_close;
extern enMaTypes Method    = ma_sma;
extern int       MaLength  = 20;

double UP[], N1[], N2[], DN[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

int init()
{
 IndicatorShortName("Directional Breakout");
 IndicatorDigits(Digits);
 SetIndexStyle(0,DRAW_HISTOGRAM);
 SetIndexBuffer(0,UP);
 SetIndexStyle(1,DRAW_HISTOGRAM);
 SetIndexBuffer(1,N1);
 SetIndexStyle(2,DRAW_HISTOGRAM);
 SetIndexBuffer(2,N2);
 SetIndexStyle(3,DRAW_HISTOGRAM);
 SetIndexBuffer(3,DN);
            timeFrame         = stringToTimeFrame(TimeFrame);
            indicatorFileName = WindowExpertName();
            returnBars        = TimeFrame=="returnBars";  if (returnBars) return(0);

 return(0);
}

int deinit()
{

 return(0);
}

int start()
{
    int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
            int limit = MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { UP[0] = MathMin(limit+1,Bars-1); return(0); }
            if (timeFrame!=Period())
            {
               limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for(int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                     UP[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",Price,Method,MaLength,0,y);
                     N1[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",Price,Method,MaLength,1,y);
                     N2[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",Price,Method,MaLength,2,y);
                     DN[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",Price,Method,MaLength,3,y);
               }
               return(0);
            }                                 

  for(int pos=limit; pos>=0; pos--)
  {
  double MA=iMA(NULL, 0, MaLength, 0, (int)Method, (int)Price, pos);
  UP[pos]=EMPTY_VALUE;
  N1[pos]=EMPTY_VALUE;
  N2[pos]=EMPTY_VALUE;
  DN[pos]=EMPTY_VALUE;
  
  if (Low[pos]>MA)
  {
   UP[pos]=2.;
  }
  else
  {
   if (High[pos]<MA)
   {
    DN[pos]=-2.;
   }
   else
   {
    N1[pos]=1.;
    N2[pos]=-1.;
   }
  }
 } 
 return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+(string)iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}